import * as d3 from 'd3';

function renderData(files) {
    // files[0] = old_id/name/new_id nodes
    // files[1] = from_id/to_id edges
    let width = 600;
    let height = 600;
    let margin = ({ top: 40, right: 40, bottom: 40, left: 40 });

    // __uncomment for the original dataset:
    // let edges = files[1].map(o => ({
    let edges = files.map(o => ({
        source: +o.from_id,
        target: +o.to_id,
        value: +o.value,
    }));

    let tooltip = d3.select('#matrix').append('div')
        .attr('class', 'tooltip')
        .style('opacity', 0);
  
  	tooltip
      // .style("position", "absolute")
        .style("text-align", "center")
        .style("padding", "5px")
        .style("font-size", "14px")
        .style("background", "rgb(231, 231, 229)")
        .style("border", "0px")
        .style("border-radius", "6px")
        .style("pointer-events", "none");

    let x = d3.scaleBand()
        .range([margin.left, margin.left + width])
        .padding(0.1);
    let y = d3.scaleBand()
        .range([margin.top, margin.top + height])
        .padding(0.1);

    let hashNode = (x, y) => x + ' --> ' + y;

    let nodeList = [];

    let nodeSet = new Set();
    edges.forEach(element => {
        nodeSet.add(element.source)
        nodeSet.add(element.target)
    });
    nodeSet.forEach(element => nodeList.push(element));
    nodeList.sort((a, b) => a - b);

    let grid = new Map();
    for (let source of nodeList) {
        for (let target of nodeList) {
            grid.set(hashNode(source, target), { x: target, y: source, value: 0 })
        }
    };

    edges.forEach(element => {
        grid.get(hashNode(element.source, element.target)).value = element.value;
        grid.get(hashNode(element.target, element.source)).value = element.value;
    });

    x.domain(nodeList);
    y.domain(nodeList);

    let svg = d3.select('#matrix')
        .append('svg')
        .attr('width', width + margin.left + margin.right)
        .attr('height', height + margin.top + margin.bottom);

    // column names
    svg.selectAll('g')
        .data(nodeList)
        .enter()
        .append('text')
        .attr('text-anchor', 'middle')
        .attr('x', margin.left / 2)
        .attr('y', d => y(d) + 4)
        .text(d => d);

    // row names
    svg.selectAll('g')
        .data(nodeList)
        .enter()
        .append('text')
        .attr('text-anchor', 'middle')
        .attr('x', d => x(d))
        .attr('y', margin.top / 1.5)
        .text(d => d);
  // title
  svg.append('text')
        .attr('x', width / 3)
        .attr('y', 13)
        .attr('fill', 'black')
        .style("font-size", "16px")
        .text('Adjacency Matrix Using D3');

    svg.selectAll('rect')
        .data(grid)
        .enter()
        .append('rect')
        .attr('x', d => x(d[1].x) - x.bandwidth() / 2)
        .attr('y', d => y(d[1].y) - x.bandwidth() / 2)
        .attr('width', x.bandwidth())
        .attr('height', y.bandwidth())
        .attr('stroke', 'lightblue')
        .attr('stroke-width', 0.5)
        .attr('fill', d => d[1].value === 0 ? '#eee' : '#f00')
        .on('mouseover', function(event, d) {
            svg.selectAll('rect')
                .attr('stroke-width', g => (g[1].x === d[1].x || g[1].y === d[1].y) ? 2 : 0.5)
            tooltip.transition().duration(150).style('opacity', 0.8)
            tooltip.html('link: ' + d[0])
                .style('left', d3.select(this).attr("cx") + 'px')
                .style('top', d3.select(this).attr("cy") + 'px')
        })
        .on('mouseout', function() {
            svg.selectAll('rect').attr('stroke-width', 0.5)
            tooltip.transition().duration(500).style('opacity', 0)
        });
}
// __uncomment for the original dataset:
// Promise.all([d3.csv(
//             'https://gist.githubusercontent.com/olga-kondr/0898f6dce8251e3e09458429d6398ed6/raw/39a9e93780f2f5ad881648ae5c5d1c0eddd2e174/fb-pages-food.csv'
//         ),
//         d3.csv(
//             'https://gist.githubusercontent.com/olga-kondr/3c239ce6bd3d44e3fa869f62564772f6/raw/d8f507c1b3f12c194e7ad949b06992d1ff718ea5/fb-pages-food-e.csv'
//         )
//     ]
//     // files[0] = old_id/name/new_id
//     // files[1] = from_id/to_id
// ).then(files => renderData(files));

d3.csv('https://gist.githubusercontent.com/olga-kondr/88320a9ad83107b2cbaf203527df1fdd/raw/f1174df24214f03f455d8d6adaad46413e369923/test.csv')
    .then(dataElements => renderData(dataElements));